#ifndef AG_DICTIONARY_H
#define AG_DICTIONARY_H

/*
 * The AGCLIB1 Class Library, Version 1.0
 * Copyright (c) Parsifal Software, 2001. All Rights Reserved.
 * http://www.parsifalsoft.com
*/

#include "agbdict.h"
#include "agref.h"
#include "agcntnr.h"

template <class Object>
class AgDictionary
: public AgCopyControl<AgDictionaryBase <Object> >
{
  typedef AgDictionaryBase<Object> DictionaryBase;
  typedef AgCopyControl<DictionaryBase> Base;
  typedef AgDictionary<Object> This;
public:
  typedef AgContents<Object, Base> Contents;
  AgDictionary() : Base(new DictionaryBase) {}
  AgDictionary(const AgContainerContents<Object> &c) : Base(new DictionaryBase(c)) {}
  AgDictionary(const This &d) : Base(d) {}

  ~AgDictionary() {}

  int intern(const Object &s) {return kernel()->intern(s);}
  int lookup(const Object &s) const {return kernel()->lookup(s);}
  int size() const {return kernel()->size();}
  This &reset() {kernel()->reset(); return *this;}
  const Contents contents() const {return Contents(*this);}

  This &operator = (const This &x) {Base::operator = (x); return *this;}
  This &operator = (const AgContainerContents<Object> &c) {*kernel() = c; return *this;}

  const Object &operator [](int x) const {return kernel()->operator [](x);}
  //int operator == (const This &x) const {return *kernel() == *x.kernel();}
  //int operator != (const This &x) const {return *kernel() != *x.kernel();}
};

template <class Object, class Container>
AgDictionary<Object> &operator += (AgDictionary<Object> &d, const Container &c) {
  int i;
  int n = c.size();
  for (i = 0; i < n; i++) d.intern(c[i]);
  return d;
}

template <class Object>
int agABTHash(const AgDictionary<Object> &d, int hashcode) {
  return agABTHashContainer(d, hashcode);
}

template <class Object>
int agABTHash(const AgDictionary<Object> &d) {
  return agABTHashContainer(d, 0);
}

#endif
